// Copyright (c) Mito
import React, { Fragment, useEffect, useState } from 'react';
import Input from '../../../elements/Input';
import MultiToggleBox from '../../../elements/MultiToggleBox';
import Select from '../../../elements/Select';
import Col from '../../../spacing/Col';
import Row from '../../../spacing/Row';
import { getAllDoesNotContainsFilterValues } from '../FilterAndSortTab/filter/utils';
var UniqueValueSortType;
(function (UniqueValueSortType) {
    UniqueValueSortType["ASCENDING_ALPHABETICAL"] = "Ascending Value";
    UniqueValueSortType["DESCENDING_ALPHABETICAL"] = "Descending Value";
    UniqueValueSortType["ASCENDING_PERCENT_OCCURENCE"] = "Ascending Occurence";
    UniqueValueSortType["DESCENDING_PERCENT_OCCURENCE"] = "Descending Occurence";
})(UniqueValueSortType || (UniqueValueSortType = {}));
const sortUniqueValueCounts = (uniqueValueCounts, uniqueValueSortType) => {
    if (uniqueValueSortType === UniqueValueSortType.ASCENDING_ALPHABETICAL) {
        return uniqueValueCounts.sort(function (a, b) {
            if (a.value < b.value) {
                return -1;
            }
            if (a.value > b.value) {
                return 1;
            }
            return 0;
        });
    }
    else if (uniqueValueSortType === UniqueValueSortType.DESCENDING_ALPHABETICAL) {
        return uniqueValueCounts.sort(function (a, b) {
            if (a.value > b.value) {
                return -1;
            }
            if (a.value < b.value) {
                return 1;
            }
            return 0;
        });
    }
    else if (uniqueValueSortType === UniqueValueSortType.ASCENDING_PERCENT_OCCURENCE) {
        return uniqueValueCounts.sort(function (a, b) {
            return a.percentOccurence - b.percentOccurence;
        });
    }
    else {
        return uniqueValueCounts.sort(function (a, b) {
            return b.percentOccurence - a.percentOccurence;
        });
    }
};
export function ValuesTab(props) {
    const [uniqueValueCounts, setUniqueValueCounts] = useState([]);
    const [uniqueValueSortType, setUniqueValueSortType] = useState(UniqueValueSortType.ASCENDING_ALPHABETICAL);
    const [searchString, setSearchString] = useState('');
    // We only load the uniqueValueCounts once on initial render of the value section
    useEffect(() => {
        void loadUniqueValueCounts();
    }, []);
    async function loadUniqueValueCounts() {
        const _uniqueValueObjs = await props.mitoAPI.getUniqueValueCounts(props.selectedSheetIndex, props.columnID);
        if (_uniqueValueObjs !== undefined) {
            /*
                Add back all of the values that were filtered out of the column, so the user can toggle
                them back on. Note that this lets users toggle them back on even if they were removed in a previous step!
            */
            const allDoesNotContainsFilters = getAllDoesNotContainsFilterValues(props.filters, props.columnMitoType);
            allDoesNotContainsFilters.forEach(key => {
                _uniqueValueObjs.push({
                    value: key,
                    percentOccurence: 0,
                    countOccurence: 0,
                    isNotFiltered: false,
                });
            });
            setUniqueValueCounts(_uniqueValueObjs);
        }
        else {
            setUniqueValueCounts([]);
        }
    }
    /*
        Gets the value of the uniqueValueCount and updates
        the filter object stored in state + sends message to backend
        to update the data.
    */
    const toggleValueFilter = (index) => {
        const value = uniqueValueCounts[index].value;
        props.toggleExclusiveFilter(value);
    };
    /*
        Helper function for getting the index of the UniqueValueCount in UniqueValueCounts from the index
        of the UniqueValueCount in the searchedUniqueValueCounts. It exploits the invariant that
        these are _unique value_ counts.
    */
    const getUniqueValueCountIndexFromSearchedAndSortedIndex = (index) => {
        const value = searchedUniqueValueCounts[index].value;
        return uniqueValueCounts.findIndex(uniqueValueCount => {
            return uniqueValueCount.value === value;
        });
    };
    /*
        Updates the isInSearch property of every uniqueValueCount. Sets it to true if the searchString is
        contained somewhere inside the uniqueValueCount. Case independant.
    */
    const searchedUniqueValueCounts = uniqueValueCounts.filter(uniqueValueCount => {
        return uniqueValueCount.value.toString().toLowerCase().search(searchString.toLowerCase()) !== -1;
    });
    const sortedUniqueValueCounts = sortUniqueValueCounts(searchedUniqueValueCounts, uniqueValueSortType);
    if (sortedUniqueValueCounts !== undefined) {
        return (React.createElement(Fragment, null,
            React.createElement(Row, { suppressTopBottomMargin: true },
                React.createElement("p", { className: 'text-header-3' }, "Unique Values (Toggle to Filter)")),
            React.createElement(Row, { justify: 'space-between' },
                React.createElement(Col, { flex: '1', offsetRight: 1 },
                    React.createElement(Input, { value: searchString, onChange: (e) => {
                            setSearchString(e.target.value);
                        }, placeholder: 'search values', width: 'block' })),
                React.createElement(Col, null,
                    React.createElement(Select, { value: uniqueValueSortType, options: Object.values(UniqueValueSortType), onChange: (newUniqueValueSortType) => {
                            setUniqueValueSortType(newUniqueValueSortType);
                        }, width: 'medium' }))),
            React.createElement(MultiToggleBox, { itemsArray: sortedUniqueValueCounts.map(uniqueValueCount => {
                    return ((React.createElement(Row, { key: uniqueValueCount.value.toString(), justify: 'space-between', align: 'center', suppressTopBottomMargin: true },
                        React.createElement(Col, { span: 16 },
                            uniqueValueCount.value.toString().length > 35 &&
                                uniqueValueCount.value.toString().substring(0, 35) + '...',
                            uniqueValueCount.value.toString().length <= 35 && uniqueValueCount.value.toString()),
                        React.createElement(Col, { span: 7, offsetRight: 1 }, uniqueValueCount.countOccurence + ' (' + uniqueValueCount.percentOccurence.toFixed(2).toString() + '%' + ')'))));
                }), itemToggleState: sortedUniqueValueCounts.map(uniqueValueCount => uniqueValueCount.isNotFiltered), toggleItemAtIndex: (searchedIndex) => {
                    const uniqueValueCountIndex = getUniqueValueCountIndexFromSearchedAndSortedIndex(searchedIndex);
                    toggleValueFilter(uniqueValueCountIndex);
                    // Manually change the toggle status so it updates instantaneously
                    uniqueValueCounts[uniqueValueCountIndex].isNotFiltered = !uniqueValueCounts[uniqueValueCountIndex].isNotFiltered;
                } })));
    }
    else {
        return (React.createElement("div", null, "There are no values to show you."));
    }
}
//# sourceMappingURL=ValuesTab.js.map